@@ -251,6 +251,12 @@ sub formbuilder_setup (@) {
 						my $name=shift;
 						length $name &&
 						$name=~/$config{wiki_file_regexp}/ &&
+						# don't allow registering
+						# accounts that look like
+						# openids, or email
+						# addresses, even if the
+						# file regexp allows it
+						$name!~/[\/:\@]/ &&
 						! IkiWiki::userinfo_get($name, "regdate");
 					},
 				);
@@ -277,7 +283,7 @@ sub formbuilder_setup (@) {
 	}
 	elsif ($form->title eq "preferences") {
 		my $user=$session->param("name");
-		if (! IkiWiki::openiduser($user)) {
+		if (! IkiWiki::openiduser($user) && ! IkiWiki::emailuser($user)) {
 			$form->field(name => "name", disabled => 1, 
 				value => $user, force => 1,
 				fieldset => "login");
@@ -319,20 +325,16 @@ sub formbuilder (@) {
 
 	if ($form->title eq "signin" || $form->title eq "register") {
 		if (($form->submitted && $form->validate) || $do_register) {
-			my $user_name = $form->field('name');
-
 			if ($form->submitted eq 'Login') {
-				$session->param("name", $user_name);
+				$session->param("name", $form->field("name"));
 				IkiWiki::cgi_postsignin($cgi, $session);
 			}
 			elsif ($form->submitted eq 'Create Account') {
-				my $email = $form->field('email');
-				my $password = $form->field('password');
-
+				my $user_name=$form->field('name');
 				if (IkiWiki::userinfo_setall($user_name, {
-					'email' => $email,
+				    	'email' => $form->field('email'),
 					'regdate' => time})) {
-					setpassword($user_name, $password);
+					setpassword($user_name, $form->field('password'));
 					$form->field(name => "confirm_password", type => "hidden");
 					$form->field(name => "email", type => "hidden");
 					$form->text(gettext("Account creation successful. Now you can Login."));
@@ -342,6 +344,7 @@ sub formbuilder (@) {
 				}
 			}
 			elsif ($form->submitted eq 'Reset Password') {
+				my $user_name=$form->field("name");
 				my $email=IkiWiki::userinfo_get($user_name, "email");
 				if (! length $email) {
 					error(gettext("No email address, so cannot email password reset instructions."));
@@ -391,9 +394,8 @@ sub formbuilder (@) {
 	elsif ($form->title eq "preferences") {
 		if ($form->submitted eq "Save Preferences" && $form->validate) {
 			my $user_name=$form->field('name');
-			my $password=$form->field('password');
-			if (defined $password && length $password) {
-				setpassword($user_name, $password);
+			if (defined $form->field("password") && length $form->field("password")) {
+				setpassword($user_name, scalar $form->field('password'));
 			}
 		}
 	}
